package it.mice.voila.runtime.springmvc.editors;

import java.beans.PropertyEditorSupport;
import java.sql.Timestamp;
import java.text.DateFormat;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.util.StringUtils;

public class CustomTimestampEditor extends PropertyEditorSupport {
	private final DateFormat[] bindDateFormat;

	private final DateFormat dateFormat;

	private final boolean allowEmpty;

	private final int exactDateLength;

	Log log = LogFactory.getLog(CustomTimestampEditor.class);

	/**
	 * Create a new CustomDateEditor instance, using the given DateFormat
	 * for parsing and rendering.
	 * <p>The "allowEmpty" parameter states if an empty String should
	 * be allowed for parsing, i.e. get interpreted as null value.
	 * Otherwise, an IllegalArgumentException gets thrown in that case.
	 * @param dateFormat DateFormat to use for parsing and rendering
	 * @param allowEmpty if empty strings should be allowed
	 */
	public CustomTimestampEditor(DateFormat[] bindDateFormats, DateFormat dateFormat, boolean allowEmpty) {
		this.bindDateFormat = bindDateFormats;
		this.dateFormat = dateFormat;
		this.allowEmpty = allowEmpty;
		this.exactDateLength = -1;
	}

	/**
	 * Create a new CustomDateEditor instance, using the given DateFormat
	 * for parsing and rendering.
	 * <p>The "allowEmpty" parameter states if an empty String should
	 * be allowed for parsing, i.e. get interpreted as null value.
	 * Otherwise, an IllegalArgumentException gets thrown in that case.
	 * <p>The "exactDateLength" parameter states that IllegalArgumentException gets
	 * thrown if the String does not exactly match the length specified. This is useful
	 * because SimpleDateFormat does not enforce strict parsing of the year part,
	 * not even with <code>setLenient(false)</code>. Without an "exactDateLength"
	 * specified, the "01/01/05" would get parsed to "01/01/0005".
	 * @param dateFormat DateFormat to use for parsing and rendering
	 * @param allowEmpty if empty strings should be allowed
	 * @param exactDateLength the exact expected length of the date String
	 */
	public CustomTimestampEditor(DateFormat[] bindDateFormats, DateFormat dateFormat, boolean allowEmpty, int exactDateLength) {
		this.bindDateFormat = bindDateFormats;
		this.dateFormat = dateFormat;
		this.allowEmpty = allowEmpty;
		this.exactDateLength = exactDateLength;
	}


	/**
	 * Parse the Date from the given text, using the specified DateFormat.
	 */
	@Override
	public void setAsText(String text) throws IllegalArgumentException {
		if (this.allowEmpty && !StringUtils.hasText(text)) {
			// Treat empty String as null value.
			setValue(null);
		}
		else if (text != null && this.exactDateLength >= 0 && text.length() != this.exactDateLength) {
			throw new IllegalArgumentException(
					"Could not parse date: it is not exactly" + this.exactDateLength + "characters long");
		}
		else {
			Exception lastEx = null;
			for (int i = 0; i < bindDateFormat.length; i++) {
				try {
					setValue(new Timestamp(this.bindDateFormat[i].parse(text).getTime()));
					return;
				}
				catch (Exception ex) {
					log.debug("Unable to parse input value: " + text + " to a valid Timestamp object by using format: " + this.bindDateFormat[i] + ". Result was: " + ex);
					lastEx = ex;
				}
			}
			if (lastEx != null) {
				throw new IllegalArgumentException("Could not parse timestamp: " + lastEx.getMessage(), lastEx);
			}
		}
	}

	/**
	 * Format the Date as String, using the specified DateFormat.
	 */
	@Override
	public String getAsText() {
		Timestamp value = (Timestamp) getValue();
		return (value != null ? this.dateFormat.format(value) : "");
	}

}
